use strict;
use Text::Xslate;


{
    package Obj;
    sub bar { 'obj ok' }
    sub method { 'Obj::method' }
}

my $tx = Text::Xslate->new(
    function => {
        f => sub { "f(@_)" },
        g => sub { "g(@_)" },
    },
);

my %vars = (
    foo      => 'bar',
    obj      => bless({}, 'Obj'),
    hashref  => { bar => "hash ok", keys => "the keys field" },
    arrayref => [10 .. 15],
    value    => { baz => "str field", 0 => "number field" },
    i        => 42,
    x        => 'x',
);

print $tx->render_string(<<'TX', \%vars);
# (1) ϐ̎Q
<: $foo :>
TX

print $tx->render_string(<<'TX', \%vars);
# (2) tB[h̎Q
<: $obj.bar     # Perl: $obj->bar() :>
<: $hashref.bar # Perl: $hashref->{bar} :>
<: $arrayref.0  # Perl: $arrayref->[0] :>
TX


print $tx->render_string(<<'TX', \%vars);
# (3) tB[h̎QƁiY̎gpj
<: $value["baz"]     # $value.baz Ɠ :>
<: $value[0]         # $value.0   Ɠ :>
TX

print $tx->render_string(<<'TX', \%vars);
# (4) \bȟĂяo
<: $obj.method(foo => 'bar') :>
<: $obj.method({ foo => [42] }) :>
TX


print $tx->render_string(<<'TX', \%vars);
# (5) \bȟĂяoigݍ݃\bhj
<: $arrayref.sort(-> $a, $b { $a <=> $b }).join(" ") :>
<: $hashref.keys().join(", ") :>
TX

print $tx->render_string(<<'TX', \%vars);
# (6) tB[h̎QƂƑgݍ݃\bȟĂяő
<: $hashref.keys   # "keys"tB[hQƂ :>
<: $hashref.keys() # nbVL[̃XgԂ :>
TX

print $tx->render_string(<<'TX', \%vars);
# (7) Perl5R̉Zq
<: defined $a
    ? "a is defined"
    : "a is undefined" :>
<: $a == "foo"  # $a"foo"Ɠǂ :>
<: $a == nil    # $a`ǂ (defined $a Ɠ) :>
<: $v != ($i + 10) # GȉZ\ :>
TX

print $tx->render_string(<<'TX', \%vars);
# (8) Perl6R̉Zq
<: "foo" ~ "bar" # A :>
<: 10 min 20 # : 10 :>
<: [10, 20, 30].reduce(-> $a, $b { $a max $b }) # 30 :>
TX

print $tx->render_string(<<'TX', \%vars);
# (9) defined-or Zq
:# $a`Ă$aA`Ȃ"default"
<: $a // "default" :>
TX

print $tx->render_string(<<'TX', \%vars);
# (10) tB^̓Kp
<: $value | dump  # dump($value) Ɠ :>
<: $x | f | g     # g(f($x)) Ɠ :>
TX
